
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

/*
 * Copyright © 2010 Gerd Kohlberger <gerdko gmail com>
 *
 * This file is part of Mousetweaks.
 *
 * Mousetweaks is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Mousetweaks is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __MT_ENUM_TYPES_H__
#define __MT_ENUM_TYPES_H__

#include <glib-object.h>

G_BEGIN_DECLS

/* enumerations from <gsettings-desktop-schemas/gdesktop-enums.h> */

#define G_DESKTOP_TYPE_MOUSE_DWELL_MODE (g_desktop_mouse_dwell_mode_get_type ())
GType g_desktop_mouse_dwell_mode_get_type (void) G_GNUC_CONST;

#define G_DESKTOP_TYPE_MOUSE_DWELL_DIRECTION (g_desktop_mouse_dwell_direction_get_type ())
GType g_desktop_mouse_dwell_direction_get_type (void) G_GNUC_CONST;

/* enumerations from "mt-common.h" */
#define MT_TYPE_DWELL_CLICK_TYPE (mt_dwell_click_type_get_type ())
GType mt_dwell_click_type_get_type (void) G_GNUC_CONST;
#define MT_TYPE_CLICK_TYPE_WINDOW_STYLE (mt_click_type_window_style_get_type ())
GType mt_click_type_window_style_get_type (void) G_GNUC_CONST;
#define MT_TYPE_CLICK_TYPE_WINDOW_ORIENTATION (mt_click_type_window_orientation_get_type ())
GType mt_click_type_window_orientation_get_type (void) G_GNUC_CONST;
G_END_DECLS

#endif /* __MT_ENUM_TYPES_H__ */

/* Generated data ends here */

